# coding=utf-8
import json
import os
from collections import OrderedDict
from axxac.path_tool import *


# 加载itest中导出的目标json格式的文件
# json_file：传入一个从itest中导出的json文件
def read_itest_json(json_file):
    if not os.path.isfile(json_file):
        return None
    else:
        file_data = open(json_file, encoding='utf-8')
        return json.load(file_data)


def get_header_list(header):
    header_list = []
    for header_item in header:
        header_list.append(
            {
                "name": str(header_item[0]),
                "value": str(header_item[1]),
                "enable": True,
                "encode": True,
                "file": False,
                "required": True,
                "valid": True
            }
        )
    return header_list


def get_assert_list(the_assert):
    assert_list = []
    for assert_item in the_assert:
        assert_list.append(
            {
                "type": "JSON",
                "expression": str(assert_item[0]),
                "expect": str(assert_item[1]),
                "description": str(assert_item[2])
            }
        )
    return assert_list


def get_extract_list(extract):
    extract_list = []
    for extract_item in extract:
        extract_list.append(
            {
                "type": "JSONPath",
                "variable": str(extract_item[0]),
                "value": "${"+str(extract_item[0])+"}",
                "expression": str(extract_item[1]),
                "multipleMatching": False,
                "valid": True
            }
        )
    return extract_list


def generate_header_item(row):
    request_item = OrderedDict([
        ("name", row[0]),
        ("value", row[1]),
        ("enable", True),
        ("encode", True),
        ("file", False),
        ("required", True),
        ("valid", True)
    ])
    return request_item


def generate_login_item(login_config):
    config = login_config['config']
    header = login_config['header']
    a_assert = login_config['assert']
    extract = login_config['extract']

    name = config[0]
    url = config[1]
    username = config[2]
    password = config[3]

    assert_list = get_assert_list(a_assert)
    extract_list = get_extract_list(extract)
    header_list = get_header_list(header)

    raw = {
        "type": 'oa',
        "rememberMe": False,
        "username": username,
        "password": password
    }

    # itest平台原因，需要保持request顺序才能正常解析
    item = OrderedDict([
        ("type", "HTTP"),
        ("name", name),
        ("enable", True),
        ("assertions", {
            "jsonPath": assert_list
        }),
        ("extract", {
            "json": extract_list
        }),
        ("url", url),
        ("method", "POST"),
        ("parameters", [{
            "type": "text",
            "enable": True,
            "contentType": "text/plain",
            "encode": True,
            "file": False,
            "required": True,
            "valid": False
        }]),
        ("headers", header_list),
        ("body", {
            "type": "Raw",
            "raw": json.dumps(raw, indent=4, ensure_ascii=False),
            "kvs": [{
                "type": "text",
                "enable": True,
                "contentType": "text/plain",
                "encode": True,
                "file": False,
                "required": True,
                "valid": False
            }],
            "format": "text",
            "json": False,
            "kV": False,
            "oldKV": False,
            "valid": True,
            "xml": False
        })
    ])
    return item


def generate_request_item(header, row, the_assert):
    raw = dict()
    for i in range(5, len(row)):
        raw[header[i]] = row[i]

    assert_list = get_assert_list(the_assert)

    # itest平台原因，需要保持request顺序才能正常解析
    item = OrderedDict([
        ("type", "HTTP"),
        ("name", row[2]),
        ("enable", True),
        ("assertions", {
            "jsonPath": assert_list
        }),
        ("url", row[1]),
        ("method", "POST"),
        ("parameters", [{
            "type": "text",
            "enable": True,
            "contentType": "text/plain",
            "encode": True,
            "file": False,
            "required": True,
            "valid": False
        }]),
        ("headers", [{
            "enable": True,
            "encode": True,
            "file": False,
            "required": True,
            "valid": False
        }]),
        ("body", {
            "type": "Raw",
            "raw": json.dumps(raw, indent=4, ensure_ascii=False),
            "kvs": [{
                "type": "text",
                "enable": True,
                "contentType": "text/plain",
                "encode": True,
                "file": False,
                "required": True,
                "valid": False
            }],
            "format": "json",
            "json": False,
            "kV": False,
            "oldKV": True,
            "valid": False,
            "xml": False
        })
    ])
    return item


def generate(result_json, login_config, output_dir):
    case_list = result_json['case_list']
    header = result_json['header']
    normal_assert = result_json['normal_assert']
    fail_assert = result_json['fail_assert']

    # 读取itest导出的json文件
    itest_json = read_itest_json(get_cur_dir()+'/itest_template.json')

    # 每一个header
    header_items = []
    for row_num in range(0, len(header)):
        header_item = generate_header_item(header[row_num])
        header_items.append(header_item)
        itest_json['scenarios'][0]['headers'] = header_items

    # 每一条case
    case_items = list()
    if login_config:
        case_items.append(generate_login_item(login_config))

    for row_num in range(1, len(case_list)):
        is_normal = row_num == 1
        the_assert = normal_assert if is_normal else fail_assert
        case_item = generate_request_item(case_list[0], case_list[row_num], the_assert)
        case_items.append(case_item)
        itest_json['scenarios'][0]['requests'] = case_items # itest格式item数组下只取1个元素

    # 写文件
    itest_json_str = json.dumps(itest_json, indent=4, ensure_ascii=False)
    output_dir = get_output_dir(output_dir)
    path = output_dir +'/itest.json'
    with open(path, 'w', encoding='utf-8') as file:
        file.write(itest_json_str)
    print("iTest JSON文件已生成："+os.path.abspath(path))

    return itest_json



# coding=utf-8
import xlrd


def get_config_value(sheet):
    row_values = sheet.row_values(1)
    return row_values


def get_header_value(sheet):
    header_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        header_rows.append(row_values)
    return header_rows


def get_token_value(sheet):
    token_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        token_rows.append(row_values)
    return token_rows


def get_assert_value(sheet):
    assert_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        assert_rows.append(row_values)
    return assert_rows


def read_excel(excel_file):
    if not excel_file:
        return None

    data = xlrd.open_workbook(excel_file)
    config_sheet = data.sheet_by_index(0)
    header_sheet = data.sheet_by_index(1)
    assert_sheet = data.sheet_by_index(2)
    extract_sheet = data.sheet_by_index(3)

    login_result_json = dict()
    login_result_json["config"] = get_config_value(config_sheet)
    login_result_json["header"] = get_header_value(header_sheet)
    login_result_json["assert"] = get_assert_value(assert_sheet)
    login_result_json["extract"] = get_token_value(extract_sheet)
    return login_result_json



# coding=utf-8
from axxac.generate_case_list import generate as clg
from axxac.generate_itest_json import generate as ijg
from axxac.read_login_config import read_excel


def execute(case_config_file, login_config_file, output_dir):
    case_list = clg(case_config_file, output_dir)
    login_config = read_excel(login_config_file)
    ijg(case_list, login_config, output_dir)


# coding=utf-8
import xlrd


def get_first_unit_value(sheet):
    return sheet.row_values(0)[0]


def get_header(sheet):
    header_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        header_rows.append(row_values)
    return header_rows


def get_params(sheet):
    params = []
    for col_num in range(sheet.ncols):
        col_values = sheet.col_values(col_num)
        remove_tail_space(col_values) # 删除每列最后由于列值个数不同而自动生成的空字符串
        param = dict()
        param['name'] = col_values[0]
        param['values'] = col_values[1:]
        params.append(param)
    return params


def remove_tail_space(array):
    new_array = array[:]
    for i in range(len(new_array)-1, -1, -1):
        if new_array[i] == '':
            array.pop()
        else:
            break


def get_assert(sheet):
    assert_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        assert_rows.append(row_values)
    return assert_rows


def read_excel(excel_file):
    data = xlrd.open_workbook(excel_file)
    name_sheet = data.sheet_by_index(0)
    url_sheet = data.sheet_by_index(1)
    header_sheet = data.sheet_by_index(2)
    params_sheet = data.sheet_by_index(3)
    assert_normal_sheet = data.sheet_by_index(4)
    assert_fail_sheet = data.sheet_by_index(5)

    result_json = dict()
    result_json["name"] = get_first_unit_value(name_sheet)
    result_json["url"] = get_first_unit_value(url_sheet)
    result_json["header"] = get_header(header_sheet)
    result_json["params"] = get_params(params_sheet)
    result_json["normal_assert"] = get_assert(assert_normal_sheet)
    result_json["fail_assert"] = get_assert(assert_fail_sheet)
    return result_json



# coding=utf-8
import os


def get_cur_dir():
    return os.path.dirname(__file__)


def get_output_dir(output_dir):
    if not output_dir or os.path.isfile(output_dir):
        output_dir = '.'
    elif not os.path.exists(output_dir):
        os.makedirs(output_dir)
    return output_dir



# coding=utf-8
from axxac import read_case_config
import time
import xlwt
from axxac.path_tool import *
import os


def write_excel_xls(path, sheet_name, value):
    index = len(value)
    workbook = xlwt.Workbook()
    worksheet = workbook.add_sheet(sheet_name)
    for i in range(0, index):
        for j in range(0, len(value[i])):
            worksheet.write(i,j, str(value[i][j]))
    workbook.save(path)


def get_normal_values(params):
    normal_values = []
    for param in params:
        values = param['values']
        normal_values.append(values[0])
    return normal_values


def generate(src_file, output_dir):
    origin_header = ['接口名称', '接口', '测试场景', '是否完成', '编写完成日期']
    header = []
    header.extend(origin_header)
    case_list = [header]
    src_json = read_case_config.read_excel(src_file)
    api_name = src_json['name']
    url = src_json['url']
    params = src_json['params']
    request_header = src_json['header']
    normal_assert = src_json['normal_assert']
    fail_assert = src_json['fail_assert']

    normal_values = get_normal_values(params)
    param_parts = [['所有参数都正确', normal_values]]  # 第一个元素为场景名

    for i in range(len(params)):
        param = params[i]
        name = param['name']
        header.append(name)

        values = param['values']
        # 遍历异常值，从第2个开始，第1个为正确值
        for j in range(1, len(values)):
            error_value = values[j]
            error_values = []
            error_values.extend(normal_values)
            error_values[i] = error_value
            param_parts.append([str(name)+'='+str(error_value), error_values])  # 第一个元素为场景名

    for i in range(len(param_parts)):
        param = param_parts[i]
        row = list()
        row.append(api_name)
        row.append(url)
        row.append(param[0])  # 场景名
        row.append('true')
        row.append(time.strftime('%Y-%m-%d', time.localtime()))
        row.extend(param[1])
        case_list.append(row)

    # 写excel
    output_dir = get_output_dir(output_dir)
    path = output_dir+'/case_list.xls'
    write_excel_xls(path, 'sheet1', case_list)
    print("\n测试用例表格已生成："+os.path.abspath(path)+"\n")

    # 加上请求头、断言信息，用于itest json生成
    result_json = {
        "case_list": case_list,
        "header": request_header,
        "normal_assert": normal_assert,
        "fail_assert": fail_assert
    }
    return result_json




# coding=utf-8
import sys
import getopt
from axxac.auto_case import execute

HELP = 'Usage:\n  axxac -c <case_config_file> -l <login_config_file> -o <output_directory>'


def run(argv, help_info):
    argv = argv[1:]
    if len(argv) == 0:
        print(help_info)
        sys.exit()
    case_config_file = ''
    login_config_file = ''
    output_dir = ''
    try:
        opts, args = getopt.getopt(argv, "hc:l:o:")
    except getopt.GetoptError:
        print(help_info)
        sys.exit()
    for opt, arg in opts:
        if opt == '-h':
            print(help_info)
            sys.exit()
        elif opt == "-c":
            case_config_file = arg
        elif opt == "-l":
            login_config_file = arg
        elif opt == "-o":
            output_dir = arg

    execute(case_config_file, login_config_file, output_dir)


if __name__ == '__main__':
    run(sys.argv, HELP)


def cmdexe():
    run(sys.argv, HELP)


from setuptools import setup, find_packages

setup(name='axxac',
      version='1.0.2',
      description='Generate case list and itest json automatically.',
      url='https://github.com/bjwuzh/autocase.git',
      author='bjwuzh',
      author_email='bjwuzh@163.com',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      entry_points={
        'console_scripts': ['axxac = axxac.main:cmdexe']
      },
      install_requires=['xlwt', 'xlrd'])
